@@ -538,8 +538,9 @@ impl MutableBuffer {
 
         let mut dst = buffer.data.as_ptr();
         for item in iterator {
+            let item = item?;
             // note how there is no reserve here (compared with `extend_from_iter`)
-            let src = item?.to_byte_slice().as_ptr();
+            let src = item.to_byte_slice().as_ptr();
             std::ptr::copy_nonoverlapping(src, dst, item_size);
             dst = dst.add(item_size);
         }
