@@ -15,78 +15,104 @@
 // specific language governing permissions and limitations
 // under the License.
 
-//! Defines kernel for length of a string array
+//! Defines kernel for length of string arrays and binary arrays
 
-use crate::{
-    array::*,
-    buffer::Buffer,
-    datatypes::{ArrowNativeType, ArrowPrimitiveType},
-};
+use crate::{array::*, buffer::Buffer, datatypes::ArrowPrimitiveType};
 use crate::{
     datatypes::{DataType, Int32Type, Int64Type},
     error::{ArrowError, Result},
 };
 
-fn unary_offsets_string<O, F>(
-    array: &GenericStringArray<O>,
-    data_type: DataType,
-    op: F,
-) -> ArrayRef
-where
-    O: StringOffsetSizeTrait + ArrowNativeType,
-    F: Fn(O) -> O,
-{
-    // note: offsets are stored as u8, but they can be interpreted as OffsetSize
-    let offsets = &array.data_ref().buffers()[0];
-    // this is a 30% improvement over iterating over u8s and building OffsetSize, which
-    // justifies the usage of `unsafe`.
-    let slice: &[O] = &unsafe { offsets.typed_data::<O>() }[array.offset()..];
+macro_rules! unary_offsets {
+    ($array: expr, $data_type: expr, $op: expr) => {{
+        let slice = $array.value_offsets();
 
-    let lengths = slice.windows(2).map(|offset| op(offset[1] - offset[0]));
+        let lengths = slice.windows(2).map(|offset| $op(offset[1] - offset[0]));
 
-    // JUSTIFICATION
-    //  Benefit
-    //      ~60% speedup
-    //  Soundness
-    //      `values` come from a slice iterator with a known size.
-    let buffer = unsafe { Buffer::from_trusted_len_iter(lengths) };
+        // JUSTIFICATION
+        //  Benefit
+        //      ~60% speedup
+        //  Soundness
+        //      `values` come from a slice iterator with a known size.
+        let buffer = unsafe { Buffer::from_trusted_len_iter(lengths) };
 
-    let null_bit_buffer = array
-        .data_ref()
-        .null_buffer()
-        .map(|b| b.bit_slice(array.offset(), array.len()));
+        let null_bit_buffer = $array
+            .data_ref()
+            .null_buffer()
+            .map(|b| b.bit_slice($array.offset(), $array.len()));
 
-    let data = unsafe {
-        ArrayData::new_unchecked(
-            data_type,
-            array.len(),
-            None,
-            null_bit_buffer,
-            0,
-            vec![buffer],
-            vec![],
-        )
-    };
-    make_array(data)
+        let data = unsafe {
+            ArrayData::new_unchecked(
+                $data_type,
+                $array.len(),
+                None,
+                null_bit_buffer,
+                0,
+                vec![buffer],
+                vec![],
+            )
+        };
+        make_array(data)
+    }};
 }
 
-fn octet_length<O: StringOffsetSizeTrait, T: ArrowPrimitiveType>(
-    array: &dyn Array,
-) -> ArrayRef
+fn length_list<O, T>(array: &dyn Array) -> ArrayRef
 where
+    O: OffsetSizeTrait,
+    T: ArrowPrimitiveType,
+    T::Native: OffsetSizeTrait,
+{
+    let array = array
+        .as_any()
+        .downcast_ref::<GenericListArray<O>>()
+        .unwrap();
+    unary_offsets!(array, T::DATA_TYPE, |x| x)
+}
+
+fn length_binary<O, T>(array: &dyn Array) -> ArrayRef
+where
+    O: BinaryOffsetSizeTrait,
+    T: ArrowPrimitiveType,
+    T::Native: BinaryOffsetSizeTrait,
+{
+    let array = array
+        .as_any()
+        .downcast_ref::<GenericBinaryArray<O>>()
+        .unwrap();
+    unary_offsets!(array, T::DATA_TYPE, |x| x)
+}
+
+fn length_string<O, T>(array: &dyn Array) -> ArrayRef
+where
+    O: StringOffsetSizeTrait,
+    T: ArrowPrimitiveType,
     T::Native: StringOffsetSizeTrait,
 {
     let array = array
         .as_any()
         .downcast_ref::<GenericStringArray<O>>()
         .unwrap();
-    unary_offsets_string::<O, _>(array, T::DATA_TYPE, |x| x)
+    unary_offsets!(array, T::DATA_TYPE, |x| x)
 }
 
-fn bit_length_impl<O: StringOffsetSizeTrait, T: ArrowPrimitiveType>(
-    array: &dyn Array,
-) -> ArrayRef
+fn bit_length_binary<O, T>(array: &dyn Array) -> ArrayRef
 where
+    O: BinaryOffsetSizeTrait,
+    T: ArrowPrimitiveType,
+    T::Native: BinaryOffsetSizeTrait,
+{
+    let array = array
+        .as_any()
+        .downcast_ref::<GenericBinaryArray<O>>()
+        .unwrap();
+    let bits_in_bytes = O::from_usize(8).unwrap();
+    unary_offsets!(array, T::DATA_TYPE, |x| x * bits_in_bytes)
+}
+
+fn bit_length_string<O, T>(array: &dyn Array) -> ArrayRef
+where
+    O: StringOffsetSizeTrait,
+    T: ArrowPrimitiveType,
     T::Native: StringOffsetSizeTrait,
 {
     let array = array
@@ -94,50 +120,59 @@ where
         .downcast_ref::<GenericStringArray<O>>()
         .unwrap();
     let bits_in_bytes = O::from_usize(8).unwrap();
-    unary_offsets_string::<O, _>(array, T::DATA_TYPE, |x| x * bits_in_bytes)
+    unary_offsets!(array, T::DATA_TYPE, |x| x * bits_in_bytes)
 }
 
-/// Returns an array of Int32/Int64 denoting the number of bytes in each string in the array.
+/// Returns an array of Int32/Int64 denoting the length of each value in the array.
+/// For list array, length is the number of elements in each list.
+/// For string array and binary array, length is the number of bytes of each value.
 ///
-/// * this only accepts StringArray/Utf8 and LargeString/LargeUtf8
+/// * this only accepts ListArray/LargeListArray, StringArray/LargeStringArray and BinaryArray/LargeBinaryArray
 /// * length of null is null.
-/// * length is in number of bytes
 pub fn length(array: &dyn Array) -> Result<ArrayRef> {
     match array.data_type() {
-        DataType::Utf8 => Ok(octet_length::<i32, Int32Type>(array)),
-        DataType::LargeUtf8 => Ok(octet_length::<i64, Int64Type>(array)),
-        _ => Err(ArrowError::ComputeError(format!(
+        DataType::List(_) => Ok(length_list::<i32, Int32Type>(array)),
+        DataType::LargeList(_) => Ok(length_list::<i64, Int64Type>(array)),
+        DataType::Utf8 => Ok(length_string::<i32, Int32Type>(array)),
+        DataType::LargeUtf8 => Ok(length_string::<i64, Int64Type>(array)),
+        DataType::Binary => Ok(length_binary::<i32, Int32Type>(array)),
+        DataType::LargeBinary => Ok(length_binary::<i64, Int64Type>(array)),
+        other => Err(ArrowError::ComputeError(format!(
             "length not supported for {:?}",
-            array.data_type()
+            other
         ))),
     }
 }
 
-/// Returns an array of Int32/Int64 denoting the number of bits in each string in the array.
+/// Returns an array of Int32/Int64 denoting the number of bits in each value in the array.
 ///
-/// * this only accepts StringArray/Utf8 and LargeString/LargeUtf8
+/// * this only accepts StringArray/Utf8, LargeString/LargeUtf8, BinaryArray and LargeBinaryArray
 /// * bit_length of null is null.
 /// * bit_length is in number of bits
 pub fn bit_length(array: &dyn Array) -> Result<ArrayRef> {
     match array.data_type() {
-        DataType::Utf8 => Ok(bit_length_impl::<i32, Int32Type>(array)),
-        DataType::LargeUtf8 => Ok(bit_length_impl::<i64, Int64Type>(array)),
-        _ => Err(ArrowError::ComputeError(format!(
+        DataType::Utf8 => Ok(bit_length_string::<i32, Int32Type>(array)),
+        DataType::LargeUtf8 => Ok(bit_length_string::<i64, Int64Type>(array)),
+        DataType::Binary => Ok(bit_length_binary::<i32, Int32Type>(array)),
+        DataType::LargeBinary => Ok(bit_length_binary::<i64, Int64Type>(array)),
+        other => Err(ArrowError::ComputeError(format!(
             "bit_length not supported for {:?}",
-            array.data_type()
+            other
         ))),
     }
 }
 
 #[cfg(test)]
 mod tests {
+    use crate::datatypes::{Float32Type, Int8Type};
+
     use super::*;
 
-    fn length_cases() -> Vec<(Vec<&'static str>, usize, Vec<i32>)> {
-        fn double_vec<T: Clone>(v: Vec<T>) -> Vec<T> {
-            [&v[..], &v[..]].concat()
-        }
+    fn double_vec<T: Clone>(v: Vec<T>) -> Vec<T> {
+        [&v[..], &v[..]].concat()
+    }
 
+    fn length_cases_string() -> Vec<(Vec<&'static str>, usize, Vec<i32>)> {
         // a large array
         let mut values = vec!["one", "on", "o", ""];
         let mut expected = vec![3, 2, 1, 0];
@@ -154,10 +189,35 @@ mod tests {
         ]
     }
 
+    macro_rules! length_binary_helper {
+        ($offset_ty: ty, $result_ty: ty, $kernel: ident, $value: expr, $expected: expr) => {{
+            let array = GenericBinaryArray::<$offset_ty>::from($value);
+            let result = $kernel(&array)?;
+            let result = result.as_any().downcast_ref::<$result_ty>().unwrap();
+            let expected: $result_ty = $expected.into();
+            assert_eq!(expected.data(), result.data());
+            Ok(())
+        }};
+    }
+
+    macro_rules! length_list_helper {
+        ($offset_ty: ty, $result_ty: ty, $element_ty: ty, $value: expr, $expected: expr) => {{
+            let array =
+                GenericListArray::<$offset_ty>::from_iter_primitive::<$element_ty, _, _>(
+                    $value,
+                );
+            let result = length(&array)?;
+            let result = result.as_any().downcast_ref::<$result_ty>().unwrap();
+            let expected: $result_ty = $expected.into();
+            assert_eq!(expected.data(), result.data());
+            Ok(())
+        }};
+    }
+
     #[test]
     #[cfg_attr(miri, ignore)] // running forever
     fn length_test_string() -> Result<()> {
-        length_cases()
+        length_cases_string()
             .into_iter()
             .try_for_each(|(input, len, expected)| {
                 let array = StringArray::from(input);
@@ -174,7 +234,7 @@ mod tests {
     #[test]
     #[cfg_attr(miri, ignore)] // running forever
     fn length_test_large_string() -> Result<()> {
-        length_cases()
+        length_cases_string()
             .into_iter()
             .try_for_each(|(input, len, expected)| {
                 let array = LargeStringArray::from(input);
@@ -188,7 +248,45 @@ mod tests {
             })
     }
 
-    fn length_null_cases() -> Vec<(Vec<Option<&'static str>>, usize, Vec<Option<i32>>)> {
+    #[test]
+    fn length_test_binary() -> Result<()> {
+        let value: Vec<&[u8]> = vec![b"zero", b"one", &[0xff, 0xf8]];
+        let result: Vec<i32> = vec![4, 3, 2];
+        length_binary_helper!(i32, Int32Array, length, value, result)
+    }
+
+    #[test]
+    fn length_test_large_binary() -> Result<()> {
+        let value: Vec<&[u8]> = vec![b"zero", &[0xff, 0xf8], b"two"];
+        let result: Vec<i64> = vec![4, 2, 3];
+        length_binary_helper!(i64, Int64Array, length, value, result)
+    }
+
+    #[test]
+    fn length_test_list() -> Result<()> {
+        let value = vec![
+            Some(vec![]),
+            Some(vec![Some(1), Some(2), Some(4)]),
+            Some(vec![Some(0)]),
+        ];
+        let result: Vec<i32> = vec![0, 3, 1];
+        length_list_helper!(i32, Int32Array, Int32Type, value, result)
+    }
+
+    #[test]
+    fn length_test_large_list() -> Result<()> {
+        let value = vec![
+            Some(vec![]),
+            Some(vec![Some(1.1), Some(2.2), Some(3.3)]),
+            Some(vec![None]),
+        ];
+        let result: Vec<i64> = vec![0, 3, 1];
+        length_list_helper!(i64, Int64Array, Float32Type, value, result)
+    }
+
+    type OptionStr = Option<&'static str>;
+
+    fn length_null_cases_string() -> Vec<(Vec<OptionStr>, usize, Vec<Option<i32>>)> {
         vec![(
             vec![Some("one"), None, Some("three"), Some("four")],
             4,
@@ -198,7 +296,7 @@ mod tests {
 
     #[test]
     fn length_null_string() -> Result<()> {
-        length_null_cases()
+        length_null_cases_string()
             .into_iter()
             .try_for_each(|(input, len, expected)| {
                 let array = StringArray::from(input);
@@ -214,7 +312,7 @@ mod tests {
 
     #[test]
     fn length_null_large_string() -> Result<()> {
-        length_null_cases()
+        length_null_cases_string()
             .into_iter()
             .try_for_each(|(input, len, expected)| {
                 let array = LargeStringArray::from(input);
@@ -233,6 +331,46 @@ mod tests {
             })
     }
 
+    #[test]
+    fn length_null_binary() -> Result<()> {
+        let value: Vec<Option<&[u8]>> =
+            vec![Some(b"zero"), None, Some(&[0xff, 0xf8]), Some(b"three")];
+        let result: Vec<Option<i32>> = vec![Some(4), None, Some(2), Some(5)];
+        length_binary_helper!(i32, Int32Array, length, value, result)
+    }
+
+    #[test]
+    fn length_null_large_binary() -> Result<()> {
+        let value: Vec<Option<&[u8]>> =
+            vec![Some(&[0xff, 0xf8]), None, Some(b"two"), Some(b"three")];
+        let result: Vec<Option<i64>> = vec![Some(2), None, Some(3), Some(5)];
+        length_binary_helper!(i64, Int64Array, length, value, result)
+    }
+
+    #[test]
+    fn length_null_list() -> Result<()> {
+        let value = vec![
+            Some(vec![]),
+            None,
+            Some(vec![Some(1), None, Some(2), Some(4)]),
+            Some(vec![Some(0)]),
+        ];
+        let result: Vec<Option<i32>> = vec![Some(0), None, Some(4), Some(1)];
+        length_list_helper!(i32, Int32Array, Int8Type, value, result)
+    }
+
+    #[test]
+    fn length_null_large_list() -> Result<()> {
+        let value = vec![
+            Some(vec![]),
+            None,
+            Some(vec![Some(1.1), None, Some(4.0)]),
+            Some(vec![Some(0.1)]),
+        ];
+        let result: Vec<Option<i64>> = vec![Some(0), None, Some(3), Some(1)];
+        length_list_helper!(i64, Int64Array, Float32Type, value, result)
+    }
+
     /// Tests that length is not valid for u64.
     #[test]
     fn length_wrong_type() {
@@ -243,7 +381,7 @@ mod tests {
 
     /// Tests with an offset
     #[test]
-    fn length_offsets() -> Result<()> {
+    fn length_offsets_string() -> Result<()> {
         let a = StringArray::from(vec![Some("hello"), Some(" "), Some("world"), None]);
         let b = a.slice(1, 3);
         let result = length(b.as_ref())?;
@@ -255,11 +393,22 @@ mod tests {
         Ok(())
     }
 
-    fn bit_length_cases() -> Vec<(Vec<&'static str>, usize, Vec<i32>)> {
-        fn double_vec<T: Clone>(v: Vec<T>) -> Vec<T> {
-            [&v[..], &v[..]].concat()
-        }
+    #[test]
+    fn length_offsets_binary() -> Result<()> {
+        let value: Vec<Option<&[u8]>> =
+            vec![Some(b"hello"), Some(b" "), Some(&[0xff, 0xf8]), None];
+        let a = BinaryArray::from(value);
+        let b = a.slice(1, 3);
+        let result = length(b.as_ref())?;
+        let result: &Int32Array = as_primitive_array(&result);
 
+        let expected = Int32Array::from(vec![Some(1), Some(2), None]);
+        assert_eq!(&expected, result);
+
+        Ok(())
+    }
+
+    fn bit_length_cases() -> Vec<(Vec<&'static str>, usize, Vec<i32>)> {
         // a large array
         let mut values = vec!["one", "on", "o", ""];
         let mut expected = vec![24, 16, 8, 0];
@@ -310,8 +459,21 @@ mod tests {
             })
     }
 
-    fn bit_length_null_cases() -> Vec<(Vec<Option<&'static str>>, usize, Vec<Option<i32>>)>
-    {
+    #[test]
+    fn bit_length_binary() -> Result<()> {
+        let value: Vec<&[u8]> = vec![b"one", &[0xff, 0xf8], b"three"];
+        let expected: Vec<i32> = vec![24, 16, 40];
+        length_binary_helper!(i32, Int32Array, bit_length, value, expected)
+    }
+
+    #[test]
+    fn bit_length_large_binary() -> Result<()> {
+        let value: Vec<&[u8]> = vec![b"zero", b" ", &[0xff, 0xf8]];
+        let expected: Vec<i64> = vec![32, 8, 16];
+        length_binary_helper!(i64, Int64Array, bit_length, value, expected)
+    }
+
+    fn bit_length_null_cases() -> Vec<(Vec<OptionStr>, usize, Vec<Option<i32>>)> {
         vec![(
             vec![Some("one"), None, Some("three"), Some("four")],
             4,
@@ -356,6 +518,22 @@ mod tests {
             })
     }
 
+    #[test]
+    fn bit_length_null_binary() -> Result<()> {
+        let value: Vec<Option<&[u8]>> =
+            vec![Some(b"one"), None, Some(b"three"), Some(&[0xff, 0xf8])];
+        let expected: Vec<Option<i32>> = vec![Some(24), None, Some(40), Some(16)];
+        length_binary_helper!(i32, Int32Array, bit_length, value, expected)
+    }
+
+    #[test]
+    fn bit_length_null_large_binary() -> Result<()> {
+        let value: Vec<Option<&[u8]>> =
+            vec![Some(b"one"), None, Some(&[0xff, 0xf8]), Some(b"four")];
+        let expected: Vec<Option<i64>> = vec![Some(24), None, Some(16), Some(32)];
+        length_binary_helper!(i64, Int64Array, bit_length, value, expected)
+    }
+
     /// Tests that bit_length is not valid for u64.
     #[test]
     fn bit_length_wrong_type() {
@@ -366,7 +544,7 @@ mod tests {
 
     /// Tests with an offset
     #[test]
-    fn bit_length_offsets() -> Result<()> {
+    fn bit_length_offsets_string() -> Result<()> {
         let a = StringArray::from(vec![Some("hello"), Some(" "), Some("world"), None]);
         let b = a.slice(1, 3);
         let result = bit_length(b.as_ref())?;
@@ -377,4 +555,19 @@ mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn bit_length_offsets_binary() -> Result<()> {
+        let value: Vec<Option<&[u8]>> =
+            vec![Some(b"hello"), Some(&[]), Some(b"world"), None];
+        let a = BinaryArray::from(value);
+        let b = a.slice(1, 3);
+        let result = bit_length(b.as_ref())?;
+        let result: &Int32Array = as_primitive_array(&result);
+
+        let expected = Int32Array::from(vec![Some(0), Some(40), None]);
+        assert_eq!(&expected, result);
+
+        Ok(())
+    }
 }
